#!/usr/bin/env python3
# vim: set shiftwidth=4 softtabstop=4:
"""Launch a docker image but bind the local directory inside the container
and define the module path automatically.
"""
import logging
import os
import pwd
import sys
from configparser import ConfigParser
from subprocess import call
from typing import List

import dockerpty
from docker.types import Mount
from psycopg2 import OperationalError, connect

from .config import ODOO_15, Config
from .docker_build import add_build_options, build_local_image, get_build_options
from .docker_client import DockerClient, modules_mount
from .docker_flake8 import apply_flake8, parser_add_flake8_group
from .docker_isort import apply_isort, parser_add_isort_group
from .docker_postgresql import POSTGRES_PASSWORD, docker_run_postgresql
from .docker_py3o import start_py3o
from .docker_pylint import apply_pylint, parser_add_pylint_group
from .parsing import apply, basic_parser

# TODO auto create list of module

_logger = logging.getLogger(__name__)

__version__ = "3.3.0"
__date__ = "2017-08-11"
__updated__ = "2022-01-27"


def main(argv=None):  # IGNORE:C0111
    """Parse arguments and launch conversion"""
    program_version = __version__
    program_build_date = str(__updated__)
    program_version_message = "%%(prog)s %s (%s)" % (
        program_version,
        program_build_date,
    )
    program_shortdesc = __doc__.split(".", maxsplit=1)[0]
    program_license = """%s

  Created by Vincent Hatakeyama on %s.
  Copyright 2017, 2018, 2019, 2020, 2021 XCG Consulting. All rights reserved.

  Licensed under the MIT License

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
""" % (
        program_shortdesc,
        str(__date__),
    )

    project_path = os.path.realpath(".")
    project_name = os.path.basename(project_path)
    if project_name == "odoo_scripts":
        logging.fatal(
            "You must run this script from the super project"
            " (./odoo_scripts/docker_dev_start.py)"
        )
        return 1

    # TODO add a way to store configuration options in a project file
    # Argument parsing
    parser = basic_parser(description=program_license, version=program_version_message)
    parser.add_argument(
        "--name",
        help="Container name to use [default: %(default)s]",
        default=project_name,
    )
    parser.add_argument(
        "--db_user", help="Database user [default: %(default)s]", default=None
    )
    parser.add_argument(
        "--db_password",
        help="Database user password [default: %(default)s]",
        default=None,
    )
    parser.add_argument(
        "--db_host",
        help="database host address",
    )
    parser.add_argument(
        "--max-cron-threads",
        help="max cron threads [default: %(default)s]",
        default=None,
    )
    parser.add_argument(
        "-d",
        "--database",
        help="Database [default: %(default)s]",
        default=None,
    )
    parser.add_argument("-p", "--dbport", help="Change database port", default=None)
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-u",
        "--update",
        help="Module to update (will also set --i18n-overwrite)"
        " [default: %(default)s]\n"
        "Options --update, --test and --test-default"
        " cannot be used at the same time",
        default=None,
    )
    group.add_argument(
        "-t",
        "--test",
        help="Modules to test (will also set --log-level=test)"
        " [default: %(default)s]\n"
        "Options --update, --test and --test-default"
        " cannot be used at the same time",
        default=None,
    )
    group.add_argument(
        "--test-default",
        help="Test all modules in module_list_test"
        " [default: %(default)s]\n"
        "Options --update, --test and --test-default"
        " cannot be used at the same time",
        action="store_true",
    )
    parser.add_argument(
        "-i",
        "--install",
        help="Module to install [default: %(default)s]",
        default=None,
    )
    parser.add_argument(
        "--without-demo",
        help="Module to avoid including demo data [default: %(default)s]",
        default=None,
    )
    parser.add_argument(
        "--create-user",
        help="Create user in database (using $user) [default: %(default)s]",
        action="store_true",
    )
    parser.add_argument(
        "--install-default",
        help=(
            "Append the module_list from setup.cfg to the modules to install"
            " [default: %(default)s]"
        ),
        action="store_true",
    )
    parser.add_argument(
        "--start-postgresql",
        help="Start a Postgresql docker",
        action="store_true",
    )
    parser_add_flake8_group(parser)
    parser_add_isort_group(parser)
    parser_add_pylint_group(parser)
    add_build_options(parser)
    parser.add_argument(
        "--odoo-help",
        help="Pass --help to odoo binary [default: %(default)s]",
        action="store_true",
    )
    parser.add_argument(
        "--stop-after-init",
        help="Pass --stop-after-init to odoo binary [default: %(default)s]",
        action="store_true",
    )
    build_group = parser.add_mutually_exclusive_group()
    build_group.add_argument(
        "--no-build",
        help="Do not build locally [default: %(default)s]",
        action="store_true",
    )
    build_group.add_argument(
        "--force-build",
        help="Force new local build [default: %(default)s]",
        action="store_true",
    )
    network_group = parser.add_mutually_exclusive_group()
    network_group.add_argument(
        "--host-network",
        help="Use host network [default: %(default)s]",
        action="store_true",
        default=True,
    )
    network_group.add_argument(
        "--docker-network",
        help=(
            "Use docker network (works better with"
            " python-netifaces installed) [default: %(default)s]"
        ),
        action="store_false",
        dest="host_network",
        default=False,
    )
    dev_option_group = parser.add_mutually_exclusive_group()
    dev_option_group.add_argument(
        "--no-dev", help="No dev options", action="store_false", dest="dev"
    )
    dev_option_group.add_argument(
        "--dev-opt",
        help="add extra dev options",
        nargs="+",
        dest="dev_opts",
        default=[],
    )
    parser.add_argument(
        "--load-language",
        help="specifies the languages for the translations you want to be"
        " loaded (also read from configuration file)",
        default=None,
        dest="LOAD_LANGUAGE",
    )
    # TODO there are more
    odoo_log_levels = ["info", "warn", "debug", "debug_sql"]
    parser.add_argument(
        "--log-level",
        help="Override odoo log level (for tests and install)",
        default=None,
        choices=odoo_log_levels,
    )
    parser.add_argument(
        "--log-handler",
        help="log handler [default: %(default)s]",
        default=None,
        action="append",
    )
    parser.add_argument(
        "--restore",
        help="Database to restore [default: %(default)s]",
        default=None,
        dest="DUMP",
    )
    parser.add_argument(
        "--config",
        help="Configuration file to use, if any [default: %(default)s]",
        default=os.path.join("conf", "dev", "odoo.conf"),
    )

    py3o_group = parser.add_mutually_exclusive_group()
    py3o_group.add_argument(
        "--start-py3o",
        help="start py3o docker [default: %(default)s]",
        action="store_const",
        default=None,
        const=1,
        dest="start_py3o",
    )
    py3o_group.add_argument(
        "--no-start-py3o",
        help="do not start py3o docker [default: %(default)s]",
        action="store_const",
        default=None,
        const=-1,
        dest="start_py3o",
    )
    parser.add_argument(
        "--odoo-sources",
        help="Odoo sources to use, if any [default: %(default)s]",
        default=None,
    )
    parser.add_argument(
        "--pre-sql-script",
        help="SQL script to run on database [default: %(default)s]",
        default=None,
        dest="PRE_SQL",
    )

    parser.add_argument(
        "--chown",
        help="run chown [default: %(default)s]",
        action="store_true",
        dest="run_chown",
    )

    # TODO detect that user is member of docker group

    # TODO add a way to add options to docker
    # TODO add a way to add arg to odoo

    nmspc = parser.parse_args(argv)
    project_name = nmspc.name
    apply(nmspc)
    db_user = nmspc.db_user
    db_password = nmspc.db_password
    use_host_network = nmspc.host_network
    _logger.debug("Use host network: %s", str(use_host_network))
    start_postgresql = nmspc.start_postgresql
    odoo_help = nmspc.odoo_help
    dev = nmspc.dev
    dev_opts = nmspc.dev_opts
    restore_filename = nmspc.DUMP
    database = nmspc.database
    load_language = ""
    installing_or_updating = False
    odoo_sources = nmspc.odoo_sources
    pre_sql_script = nmspc.PRE_SQL
    run_chown = nmspc.run_chown

    if restore_filename:
        if not database:
            _logger.fatal("No database name given for restore")
            return 13

    c = Config()

    if not db_user and c.db_user is not None:
        db_user = c.db_user
    if not db_password and c.db_password is not None:
        db_password = c.db_password
    if c.load_language is not None:
        load_language = c.load_language

    odoo_type = c.odoo_type
    image = c.local_image_name
    postgresql_version = c.postgresql_version
    module_list = c.module_list
    if nmspc.start_py3o:
        _logger.debug("Command line start py3o %d", nmspc.start_py3o)
        start_py3o_stack = nmspc.start_py3o == 1
    else:
        start_py3o_stack = c.start_py3o
        _logger.debug("No command line start py3o %d", start_py3o_stack)

    _logger.debug("Docker image: %s", image)
    # detect if docker image already exists
    client = DockerClient.client
    if not nmspc.no_build:
        failed = build_local_image(get_build_options(nmspc), nmspc.force_build)
        if failed:
            return failed

    # options is only used with subprocess call
    options = {
        "name": project_name,
        "hostname": project_name,
        "tty": True,
        "extra_hosts": dict(),
        "ports": dict(),
        "environment": dict(),
    }
    if not use_host_network:
        options["ports"][8069] = 8069
    mounts: List[Mount] = []
    # TODO handle other version of odoo (base image need to be changed too)
    coverage = (nmspc.test or nmspc.test_default) and odoo_type in (
        "odoo11",
        "odoo13",
        ODOO_15,
    )
    arg = ["coverage-start" if coverage else "start"]
    if db_password:
        arg.append("--db_password %s" % db_password)
    if nmspc.update:
        arg.extend(("-u", nmspc.update))
        arg.append("--i18n-overwrite")
        installing_or_updating = True
    if nmspc.test or nmspc.test_default:
        arg.append("--test-enable")
    if nmspc.test:
        arg.extend(("-u", nmspc.test))
        if odoo_type == "odoo7":
            arg.append("--log-level=test")
        installing_or_updating = True
    if nmspc.test or nmspc.stop_after_init:
        arg.append("--stop-after-init")
    if nmspc.test_default:
        test_modules = c.module_list_tests
        str_modules = ",".join(test_modules)
        arg.extend(("-u", str_modules))
        if odoo_type == "odoo7":
            arg.append("--log-level=test")
        arg.append("--stop-after-init")
        installing_or_updating = True
    if database:
        arg.extend(("-d", database))
    if nmspc.install or nmspc.install_default:
        installing_or_updating = True
        modules_to_install = []
        if nmspc.install:
            modules_to_install.extend(nmspc.install.split(","))
        if nmspc.install_default:
            modules_to_install.extend(module_list)
        if modules_to_install:
            arg.append("-i")
            arg.append(",".join(modules_to_install))
    if nmspc.without_demo:
        arg.append("--without-demo")
        arg.append(nmspc.without_demo)
    if nmspc.max_cron_threads:
        arg.append("--max-cron-threads=%s" % nmspc.max_cron_threads)
    if nmspc.log_level:
        arg.append("--log-level")
        arg.append(nmspc.log_level)
    if nmspc.log_handler:
        for lh in nmspc.log_handler:
            arg.append("--log-handler")
            arg.append(lh)
    if nmspc.LOAD_LANGUAGE:
        load_language = nmspc.LOAD_LANGUAGE
    if load_language and installing_or_updating:
        arg.append("--load-language")
        arg.append(load_language)

    # auto detect local ip
    if use_host_network:
        local_ip = "127.0.0.1"
        options["network_mode"] = "host"
    else:
        local_ip = None
        try:
            from netifaces import AF_INET, ifaddresses  # apt python3-netifaces

            iface_name = "docker0"
            addresses = [
                i["addr"]
                for i in ifaddresses(iface_name).setdefault(
                    AF_INET, [{"addr": "No IP addr"}]
                )
            ]
            if addresses:
                local_ip = addresses[0]
        except ImportError:
            _logger.warn(
                "Consider installing python netifaces" " to ease local IP detection"
            )
        if not local_ip:
            import socket

            _logger.info("Contacting Google Public DNS to find our IP")
            local_ip = [
                (s.connect(("8.8.8.8", 53)), s.getsockname()[0], s.close())
                for s in [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]
            ][0][1]

        _logger.debug("IP found %s", local_ip)

    # auto detect local conf
    local_conf_path = nmspc.config
    user = db_user
    password = db_password
    if os.path.isfile(local_conf_path):
        _logger.info("Local configuration file found: %s", local_conf_path)
        odoo_conf_file = (
            "/etc/odoo/odoo.conf"
            if odoo_type in ("odoo11", "odoo12", "odoo13", ODOO_15)
            else "/opt/odoo/etc/odoo.conf"
        )
        mounts.append(
            Mount(
                odoo_conf_file,
                os.path.join(project_path, local_conf_path),
                "bind",
            )
        )
        cp_local = ConfigParser()
        cp_local.read(local_conf_path)
        if not user:
            user = cp_local.get("options", "db_user", fallback=None)
        if not password:
            password = cp_local.get("options", "db_password", fallback=None)
        # doing this avoid having to change the value of redis_host
        # used by smile_redis_session_store
        redis_host = (
            cp_local.has_section("options")
            and cp_local.has_option("options", "redis_host")
            and cp_local.get("options", "redis_host")
        ) or None
        if redis_host:
            options["extra_hosts"][redis_host] = local_ip
    else:
        _logger.info("No configuration file at: %s", local_conf_path)
    # default values if nothing else
    if not user:
        # Odoo uses default if nothing is set (including in PGUSER env var)
        if start_postgresql:
            _logger.info("No user defined, using odoo when starting postgres")
            user = "odoo"
        else:
            # use environment variable
            user = os.getenv("PGUSER")
            # defaulting to odoo
            if not user:
                user = "odoo"
    arg.append("--db_user")
    arg.append(user)

    # data volume handling
    base_data_volume_name = "{}_data".format(project_name)
    if odoo_type == "odoo7":
        # this path is in odoo code, the last part is the system user
        session_volume_name = "{}{}".format(base_data_volume_name, "_sessions")
        target = "/tmp/oe-sessions-{}".format("odoo")
        client.create_volume(session_volume_name, {"mounted in": target})
        mounts.append(Mount(target, session_volume_name))
        # by default odoo store attachments in database but it can be set on
        # disk. mount the volume in all cases.
        # XXX data is the path set in the database (in the form file:///data)
        #  it should be read from there, in that case target directory needs
        #  to include the database name.
        #  our databases only use file:///data so this should be fine
        target = (
            "/usr/local/lib/python2.7/dist-packages/"
            "openerp-7.0-py2.7.egg/openerp/data"
        )
        filestore_volume_name = "{}{}".format(base_data_volume_name, "_filestore")
        client.create_volume(filestore_volume_name, {"mounted in": target})
        mounts.append(target, filestore_volume_name)
    else:
        data_volume_path = "/mnt/data"
        mounts.append(Mount(data_volume_path, base_data_volume_name))
        arg.append("--data-dir")
        arg.append(data_volume_path)

    # make sure the permission in the volumes are correct
    if run_chown:
        chown_directories = ["/mnt"]
        if odoo_type not in ("odoo7", "odoo8"):
            chown_directories.append("/var/lib/odoo")
        _logger.info("chown container’s %s to Odoo user", ", ".join(chown_directories))
        client.containers.run(
            image,
            command=["odoo", "--recursive"] + chown_directories,
            remove=True,
            entrypoint="/bin/chown",
            mounts=mounts,
            user="root",
        )

    # avoid the duplication of unbind volumes with all addons
    if odoo_type == "odoo7":
        extra_volumes = [
            "/opt/odoo/additional_addons",
            "/opt/odoo/var",
            # not used but declared as a mount point in Dockerfile :-(
            "/opt/odoo/data",
        ]
    elif odoo_type == "odoo8":
        extra_volumes = [
            "/opt/odoo/additional_addons",
            "/opt/odoo/var",
            "/opt/odoo/data",
        ]
    elif odoo_type == "odoo10":
        extra_volumes = [
            "/opt/odoo/additional_addons",
            "/opt/odoo/var",
            "/opt/odoo/data",
        ]
    else:
        extra_volumes = ["/var/lib/odoo"]
    for extra_volume in extra_volumes:
        volume_name = "{}_{}".format(project_name, extra_volume.replace("/", "_"))
        DockerClient.create_volume(
            volume_name,
            {
                "mounted in": extra_volume,
                "comment": "only used to avoid creating too many volumes",
            },
        )
        mounts.append(Mount(extra_volume, volume_name))

    if start_py3o_stack:
        start_py3o()
        # provide a couple of handy dns name for py3o
        options["extra_hosts"]["py3o"] = local_ip
        options["extra_hosts"]["py3o-fusion-server"] = local_ip

    # Database connection

    dbport = None
    if nmspc.dbport:
        dbport = str(nmspc.dbport)
        arg.append("--db_port")
        arg.append(dbport)

    db_host = nmspc.db_host
    # Check that connection can be done, try to create user if asked to
    # TODO: handle the case where the database is still starting up
    # TODO: (and remove the sleep)
    if not odoo_help:
        if start_postgresql:
            if db_host:
                _logger.warning("Ignoring given db_host when starting postgres")
            if dbport:
                _logger.warning("Ignoring given db_port when starting postgres")
            db_host = "/var/run/postgresql"
            dbport = 5432
            # Use socket rather than export port (avoid having to find a
            # free port number)
            pg, stop_postgresql, socket_path = docker_run_postgresql(
                project_name, postgresql_version, host_pg_port=None
            )
            mounts.append(Mount(db_host, socket_path, "bind"))
        _logger.info("Testing database connection on base postgres")
        try:
            if start_postgresql:
                connect(
                    user=user,
                    password=password,
                    database="postgres",
                    host=socket_path,
                    port=dbport,
                )
                _logger.info("Connection successful")
            else:
                # psql uses PGHOST if provided and a socket otherwise
                if db_host:
                    pass
                elif "PGHOST" in os.environ:
                    db_host = os.getenv("PGHOST")
                    _logger.debug("Using PGHOST=%s", db_host)
                else:
                    # use socket, trying to guess it
                    # db_host = "/var/run/postgresql"
                    # _logger.debug("Defaulting to host %s", db_host)
                    db_host = local_ip
                if db_host.startswith("/"):
                    # mount the path inside the docker
                    mounts.append(Mount(db_host, db_host, "bind"))
                connect(
                    user=user,
                    password=password,
                    database="postgres",
                    host=db_host,
                    port=dbport,
                )

        except OperationalError as exception:
            if nmspc.create_user:
                _logger.debug("Cannot connect to database with user %s", user)
                _logger.debug(exception)
                _logger.info("Creating user %s", user)
                if start_postgresql:
                    connection = connect(
                        user="postgres",
                        database="postgres",
                        host=db_host,
                        port=5432,
                        password=POSTGRES_PASSWORD,
                    )
                else:
                    loginname = pwd.getpwuid(os.getuid())[0]
                    # use socket to create user
                    # TODO test if removing more arguments works
                    connection = connect(user=loginname, database="postgres")
                with connection.cursor() as cursor:
                    # not injection safe but you are on your own machine
                    # with already full access to db
                    cursor.execute(
                        "CREATE ROLE %s LOGIN CREATEDB PASSWORD %%s" % user,
                        (password,),
                    )
                connection.commit()
                connection.close()
            else:
                _logger.critical("Cannot connect to database with user %s", user)
                _logger.critical(exception)
                _logger.info("You can add the --create-user argument to create it")
                return 16

    arg.append("--db_host")
    arg.append(db_host)

    # --- restore ---
    # TODO find out why odoo_help would stop us from restoring
    if start_postgresql and not odoo_help and restore_filename:
        restore_basename = os.path.basename(restore_filename)
        _logger.info("Copying dump file in docker")
        inside_restore_filename = "/tmp/{}".format(restore_basename)
        DockerClient.put_file(restore_filename, pg, inside_restore_filename)
        _logger.info("Creating database %s for %s", database, user)
        pg.exec_run(["createdb", "-U", user, database])
        _logger.info("Restoring database %s", database)
        restore, output = pg.exec_run(
            [
                "pg_restore",
                "-U",
                user,
                "--no-owner",
                "-d",
                database,
                inside_restore_filename,
            ]
        )
        if not restore:
            return 15
        _logger.info("Removing dump file in docker")
        pg.exec_run(["rm", inside_restore_filename])

    if not start_postgresql and not odoo_help and restore_filename:
        _logger.info("Creating database %s", database)
        createdb = call(["createdb", "-U", user, database])
        if not createdb:
            return 17
        _logger.info("Restoring database %s", database)
        restore = call(
            ["pg_restore", "-U", user, "-O", "-d", database, restore_filename]
        )
        if not restore:
            return 15

    # --- Pre SQL script ---
    if pre_sql_script:
        if start_postgresql:
            pre_sql_script_basename = os.path.basename(pre_sql_script)
            _logger.info("Copying SQL script %s in docker", pre_sql_script_basename)
            inside_pre_sql_script = "/tmp/{}".format(pre_sql_script_basename)
            client.put_file(pre_sql_script, pg, inside_pre_sql_script)
            _logger.info(
                "Running SQL script %s in %s",
                pre_sql_script_basename,
                database,
            )
            exit_code, output = pg.exec_run(
                [
                    "psql",
                    "-U",
                    user,
                    "-d",
                    database,
                    "-f",
                    inside_pre_sql_script,
                ]
            )
            if exit_code > 0:
                _logger.fatal("Error when running SQL script %s", pre_sql_script)
                return 16
            _logger.info("Removing SQL script %s in docker", pre_sql_script)
            # remove script
            pg.exec_run(["rm", inside_pre_sql_script])
        else:
            _logger.info("Running SQL script %s in %s", pre_sql_script, database)
            script = call(["psql", "-U", user, "-d", database, "-f", pre_sql_script])
            if script > 0:
                _logger.fatal("Error when running SQL script %s", script)
                return 16

    # volume magic
    modules = c.modules
    _logger.debug("addon modules: %s", ",".join(modules))
    target_module_directory = "/mnt/addons"
    mounts.extend(modules_mount(modules, project_path, target_module_directory)[0])
    all_addons_dir = [target_module_directory]
    if odoo_type in ("odoo7", "odoo8"):
        all_addons_dir.append("/opt/odoo/sources/odoo/addons")
    if all_addons_dir:
        arg.append("--addons-path")
        arg.append(",".join(all_addons_dir))

    apply_flake8(nmspc, odoo_type)
    apply_isort(nmspc, odoo_type)
    apply_pylint(nmspc)

    # developer mode options
    if dev:
        if odoo_type in ("odoo10", "odoo11", "odoo13", ODOO_15):
            # automatically add reload if not already asks for
            if not any(opt in dev_opts for opt in ("all", "reload")):
                dev_opts.append("reload")
            # XXX pdb should not be run if not interactive
            # add pdb if no pubd, wdb, ipdb, pdb or all
            # the list of available options differs by odoo version and can be
            # seen with odoo --help, available with --odoo-help
            if not any(
                debugger in dev_opts
                for debugger in ("pubd", "wdb", "ipdb", "pdb", "all")
            ):
                dev_opts.append("pdb")
            arg.append(f"--dev={','.join(set(dev_opts))}")
        if odoo_type in ("odoo8",):
            arg.append("--auto-reload")

    if odoo_help:
        arg.append("--help")

    # bind Odoo sources
    if odoo_sources:
        _logger.info("Binding Odoo sources at %s", odoo_sources)
        if odoo_type in ("odoo10", "odoo11", "odoo13", ODOO_15):
            # Image 11 uses python 3.5 if based on xenial, or 3.6 for bionic
            # 13 uses bionic
            if odoo_type == "odoo10":
                pythons = ("python2.7",)
            elif odoo_type == "odoo11":
                pythons = ("python3.5", "python3.6")
            elif odoo_type == "odoo13":
                pythons = ("python3.8",)
            elif odoo_type == ODOO_15:
                pythons = ("python3.9",)
            else:
                pythons = ("python3.6",)
            for python in pythons:
                mounts.append(
                    Mount(
                        "/usr/local/lib/%s/dist-packages/odoo" % python,
                        os.path.join(odoo_sources, "odoo"),
                        "bind",
                    )
                )
                mounts.append(
                    Mount(
                        "/usr/local/lib/%s/dist-packages/odoo/addons" % python,
                        os.path.join(odoo_sources, "addons"),
                        "bind",
                    )
                )
                mounts.append(
                    Mount(
                        "/usr/local/lib/%s/dist-packages/odoo/addons/base" % python,
                        os.path.join(odoo_sources, "odoo", "addons", "base"),
                        "bind",
                    )
                )
        elif odoo_type in ("odoo7", "odoo8"):
            mounts.append(Mount("/opt/odoo/sources/odoo", odoo_sources, "bind"))
        else:
            raise Exception("Unexpected odoo_type when binding sources: %s" % odoo_type)

    # avoid chowning at each run, faster but can cause trouble
    # newer odoo images does not use this environment variable anymore
    options["environment"]["ODOO_CHOWN"] = "false"

    if coverage:
        if nmspc.test_default:
            source_files = target_module_directory
        else:
            # TODO when testing base modules, coverage will not be shown
            source_files = ",".join(
                [
                    "{}/{}".format(target_module_directory, module)
                    for module in nmspc.test.split(",")
                ]
            )
        options["environment"]["COVERAGE_RUN_OPTIONS"] = (
            "--source=" + source_files + " --omit=*/__manifest__.py"
        )

    interactive = True
    if interactive:
        options["stdin_open"] = True
    odoo_container = client.containers.create(
        image, command=arg, mounts=mounts, **options
    )
    _logger.info("Starting Odoo")
    if interactive:
        dockerpty.start(client.api, odoo_container.id)
        # already stopped but this is the easiest way to get the return code
        result_dict = odoo_container.wait()
        result = result_dict["StatusCode"]
    else:
        # XXX untested
        #  that might need a way to pass any signal to the container
        result = odoo_container.start()
    # only remove after getting the return code
    odoo_container.remove()
    return result


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))
